@@ -1,18 +1,18 @@
-/*  Tarlz - Archiver with multimember lzip compression
-    Copyright (C) 2013-2019 Antonio Diaz Diaz.
+/* Tarlz - Archiver with multimember lzip compression
+   Copyright (C) 2013-2020 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #define _FILE_OFFSET_BITS 64
@@ -32,6 +32,8 @@
 
 namespace {
 
+const CRC32 crc32c( true );
+
 unsigned decimal_digits( unsigned long long value )
   {
   unsigned digits = 1;
@@ -280,7 +282,8 @@ void Extended::fill_from_ustar( const Tar_header header )
 
 
 /* Returns file size from record or from ustar header, and resets file_size_.
-   Used for fast parsing of headers in uncompressed archives. */
+   Used for fast parsing of headers in uncompressed archives.
+*/
 long long Extended::get_file_size_and_reset( const Tar_header header )
   {
   const long long tmp = file_size_;
